package de.beyondjava.merge_jsf_jsr303

import de.beyondjava.simpleClassParser.ClassDefinition
import de.beyondjava.simpleClassParser.SimpleBeanParser
import de.beyondjava.simpleClassParser.VariableDefinition

class Merger {
	/**
	 * Analyzes every input text of every JSF file in a folder.
	 * If the bean variable of an input field bears a bean validation annotation,
	 * the JSF file is modified to contain the same information.
	 * @param p_sourceFolder The folder containing the beans.
	 * @param p_jsfFolder The folder containing the *.xhtml files.
	 * @param p_targetJSFFolder The folder to store the modified jsf files in.
	 */
	void mergeBeanValidationIntoJSF(String p_sourceFolder, String p_jsfFolder, String p_targetJSFFolder)
	{
		SimpleBeanParser parser = new SimpleBeanParser().parse(p_sourceFolder)
		List<ClassDefinition> beans = parser.beans
		List<VariableDefinition> vars = parser.vars
		enrichEachJSFFile(p_jsfFolder, p_targetJSFFolder, vars)
	}

	private void enrichEachJSFFile(String p_jsfFolder, String p_targetJSFFolder, List<VariableDefinition> p_vars)
	{
		File g;
		List<File> files = []
		p_jsfFolder = new File(p_jsfFolder).absolutePath
		new File(p_jsfFolder).eachFileRecurse({if (it.isFile() && it.name.endsWith(".xhtml")) files = files + it })
		List<File> targetFiles = files.collect({File f -> new File(f.absolutePath.replace(p_jsfFolder, p_targetJSFFolder))})
		files.eachWithIndex({File source, int index -> enrichJSFFile(source, targetFiles[index], p_vars) })
	}

	private void enrichJSFFile(File p_sourceFile, File p_targetFile, List<VariableDefinition> vars)
	{
		def jsf = new XmlParser().parse(p_sourceFile);
		def inputs = jsf['**']
		def inputFields = inputs.findAll{ it.name().toString().endsWith("inputText")}
		inputFields.each{ addTags(it, vars)}
		def writer = new StringWriter()
		new XmlNodePrinter(new PrintWriter(writer)).print(jsf)
		def result = writer.toString()
		p_targetFile.text = result
	}

	private void addTags(def p_node, List<VariableDefinition> vars)
	{
		String value = p_node.@value
		if (value[0..1]=="#{" && value[-1]=="}")
		{
			String potentialVariable = value[2..-2]
			VariableDefinition match = vars.find{VariableDefinition v -> v.name == potentialVariable}
			if (match)
			{
				if (match.annotations.any{it=="@NotNull"})
				{
					p_node.@required="true"
				}
				String sizeTag = match.annotations.find{it=~ /@Size(\s)*\((.)*\)/}
				if (sizeTag)
				{
					sizeTag=sizeTag.replaceAll(/@Size(\s)*\(/, "").trim()

						String[] sizes = sizeTag[0..-2].split(",")
						sizes.each{String s -> if (s.trim().startsWith("max")) addMaxLengthTag(p_node, s) }
						sizes.each{String s -> if (s.trim().startsWith("min")) p_node.@required="true" }


				}
			}
		}
	}

	private void addMaxLengthTag(def p_node, String p_maxLength)
	{
		String[] terms = p_maxLength.split("=")
		String maxLength = terms[1].trim()
		int iMaxLength = new Integer(maxLength).intValue()
		if (p_node.@maxlength)
		{
			if ("$iMaxLength" != p_node.@maxlength)
			{
				println "Warning: Bean Validation Constraint differs from JSF maxlength attribute in input field ${p_node.@value}: ${p_node.@maxlength} (JSF) vs. $iMaxLength (bean annotation)"
			}
		}
		else p_node.@maxlength="$iMaxLength"
	}
}
